/***********************************
128 by 64 LCD Oscilloscope - Default
By Bob Davis
Uses Universal 8bit Graphics Library, http://code.google.com/p/u8glib/
  Copyright (c) 2012, olikraus@gmail.com  All rights reserved.

********************************************/
#include "U8glib.h"

// 8Bit Com: D0..D7: 8,9,10,11,4,5,6,7 en=18, di=17,rw=16
U8GLIB_ST7920_128X64_4X u8g(8, 9, 10, 11, 4, 5, 6, 7, 18, 17, 16); 

int Sample[128];
int Input=0;
int OldInput=0;
int rate=0;
int StartSample=0;
int EndSample=0;
int SampleTime=0;

void u8g_prepare(void) {
  u8g.setFont(u8g_font_6x10);
  u8g.setFontRefHeightExtendedText();
  u8g.setDefaultForegroundColor();
  u8g.setFontPosTop();
}
void DrawMarkers(void) {
  u8g.drawFrame (0,0,128,64);
  u8g.drawPixel (25,16);
  u8g.drawPixel (50,16);
  u8g.drawPixel (100,16);
  u8g.drawPixel (25,32);
  u8g.drawPixel (50,32);
  u8g.drawPixel (100,32);
  u8g.drawPixel (25,48);
  u8g.drawPixel (50,48);
  u8g.drawPixel (100,48);
}

void sample_data(void){
// wait for a trigger of a positive going input
// Input=analogRead(A0);
  while (Input < OldInput){
    OldInput=analogRead(A0);
    Input=analogRead(A0);
  }
// collect the analog data into an array
// do not do division by 10.24 here, it makes it slower!
  StartSample = micros();
  for(int xpos=0; xpos<100; xpos++) {
    Sample[xpos]=analogRead(A0);
  } 
    EndSample = micros();
}  

void draw(void) {
  char buf[12];
  u8g_prepare();
  DrawMarkers(); 
// display the collected analog data from array
// Sample/10.24 because 1024 becomes 100 = 5 volts
  for(int xpos=1; xpos<99; xpos++) {
    u8g.drawLine (xpos, Sample[xpos]/10.24, xpos+1, Sample[xpos+1]/10.24);
  }  
  SampleTime=EndSample-StartSample;
  u8g.drawStr(100, 8, "Time");
  u8g.drawStr(100, 18, itoa(SampleTime, buf, 10));
}

void setup(void) {
  // assign default color value
  if ( u8g.getMode() == U8G_MODE_R3G3B2 ) 
    u8g.setColorIndex(255);     // RGB=white
  else if ( u8g.getMode() == U8G_MODE_GRAY2BIT )
    u8g.setColorIndex(3);       // max intensity
  else if ( u8g.getMode() == U8G_MODE_BW )
    u8g.setColorIndex(1);       // pixel on, black
}

void loop(void) {
// collect the data
  sample_data();
// show collected data 
  u8g.firstPage();  
  do { draw(); }  
  while( u8g.nextPage() );
// rebuild the picture after some delay
  delay(100);
}


