;**********************************************************************
;   This file is a basic code template for assembly code generation   *
;   on the PIC12F675. This file contains the basic code               *
;   building blocks to build upon.                                    *
;                                                                     *
;   Refer to the MPASM User's Guide for additional information on     *
;   features of the assembler (Document DS33014).                     *
;                                                                     *
;   Refer to the respective PIC data sheet for additional             *
;   information on the instruction set.                               *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Filename:	    SOLAR CONTROL,ASM                                 *
;    File Version:                                                    *
;    DECEMBER 5, 2013                                                 *
;    Author:       RON NEWTON                                         *
;    Company:                                                         *
;                                                                     *
;                                                                     *
;**********************************************************************
;                                                                     *
;    Files Required: P12F675.INC                                      *
;                                                                     *
;**********************************************************************
;                                                                     
;    Notes:     
;	1	VDD
;	2	GPIO 5 NC
;	3	GPIO 4 LED RESISTOR
;	4	GPIO 3 LED
;	5	GPIO 2 GATE
;	6	GPIO 1 ANALOG IN BATTERY
;	7	GPIO 0 ANALOG IN SOLAR
;	8	VSS                                             
;                                                                     
;**********************************************************************

	list      p=12f675           ; list directive to define processor
	#include <p12f675.inc>        ; processor specific variable definitions

	RADIX		DEC	;DEC IS DEFAULT # FORMAT

	
    __CONFIG _CP_OFF & _BODEN_OFF & _PWRTE_OFF & _MCLRE_OFF &_WDT_OFF & _INTRC_OSC_NOCLKOUT

	errorlevel  -302              ; suppress message 302 from list file



; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.




;**********************************************************************
		ORG     0x000             ; processor reset vector
		goto    AWAYWEGO          ; go to beginning of program
	

		ORG     0x004             ; interrupt vector location
		movwf   w_temp            ; save off current W register contents
		movf	STATUS,w          ; move status register into W register
		movwf	status_temp       ; save off contents of STATUS register
INT0	DECFSZ	TIMER1,1		  ; COUNTER FOR ONE SECOND PULSE
		GOTO	INT1			  ;INTERRUPTING EVER 4 MILISECONDS
		MOVLW	.16
		MOVWF	TIMER1

		DECFSZ	SECONDS,1			;SECONDS COUNTER
		GOTO	INT0
		MOVLW	.60					;RESET SECONDS
		MOVWF	SECONDS
		DECFSZ	MINUTES,1			;MINUTE COUNTER
		GOTO	INT0
		MOVLW	.10					;RESET MINUTES
		MOVWF	MINUTES

ABOVE_14.25							;CHECKING BATTERY VOLTAGE
		CALL	TURN_OFF
		MOVLW	.50					;LET REST FOR 10 SECONDS
		MOVWF	COUNTER3
		DECFSZ	COUNTER1,1
		GOTO	$-1
		DECFSZ	COUNTER2,1
		GOTO	$-3
		DECFSZ	COUNTER3,1
		GOTO	$-5
		CALL	MEASURE_BATTERY
  		MOVLW	.1	^0FFH			;ABOVE
		ADDWF	BATTERY_HI,0
		BTFSC	3,0
		GOTO	ABOVE1
		MOVLW	.6	 				;1.400 IF ABOVE  1.4  .31  .6 FOR .909 FACTOR
		SUBWF	BATTERY_LOW,0
		BTFSS	3,0
		GOTO	INT1				;NOT ABOVE CONTINUE

ABOVE1	CALL	TURN_OFF			;WATING FOR BATTERY TO DISCHARGE
		CALL	MEASURE_BATTERY	
		MOVLW	.1					;IF BELOW 1
		SUBWF	BATTERY_HI,0
		BTFSS	3,0
		GOTO	INT1
		MOVLW	.1
		SUBWF	BATTERY_HI,0
		BTFSS	3,2
		GOTO	ABOVE1
 		MOVLW	.251					;13.5 VOLTS  JUMP IF EQUAL OR BELOW 21  251 FOR .909 FACTOR
		SUBWF	BATTERY_LOW,0
		BTFSC	3,0
		GOTO	ABOVE1				;KEEP CHECKING UNTIL BELOW 13.5 VOLTS
		CALL	TURN_ON

INT1	BCF		INTCON,T0IF		  ;RESET TOIF FLAG
		movf    status_temp,w     ; retrieve copy of STATUS register
		movwf	STATUS            ; restore pre-isr STATUS register contents
		swapf   w_temp,f
		swapf   w_temp,w          ; restore pre-isr W register contents
		retfie                    ; return from interrupt


; these first 4 instructions are not required if the internal oscillator is not used
main
		call    0x3FF             ; retrieve factory calibration value
		bsf     STATUS,RP0        ; set file register bank to 1 
		movwf   OSCCAL            ; update register with factory cal value 
		bcf     STATUS,RP0        ; set file register bank to 0

;------------------------------------------------------------------------------
;       Bit Equates 
;****************************************************************************


    CBLOCK 0x20 ; Define GPR variable register locations

BATTERY_HI
BATTERY_LOW
SOLAR_HI
SOLAR_LOW
COUNTER
COUNTER1
COUNTER2
COUNTER3
ACCaLO
ACCaMID
ACCaHI 
ACCbLO
ACCbMID
ACCbHI
ACCdLO
ACCdMID
ACCdHI
TIMER1
TIMER2
TIMER3
SECONDS
MINUTES
w_temp        
status_temp   

    ENDC

#DEFINE		GREEN	  	 GPIO,4
#DEFINE		RED			 GPIO,5
#DEFINE		POWER		 GPIO,2



AWAYWEGO
		  BCF	STATUS, RP0			;PAGE 0
 		  CLRF	GPIO				;CLEAR GPIO
		  MOVLW	07H					;MAKING CMCON LOW POWER
		  MOVWF	CMCON				;LOAD
          BSF   STATUS, RP0        	;JUMP TO PAGE 1
		  MOVLW	B'00000011'			;MAKING RP0 RP1 INPUT
		  MOVWF	TRISIO				;LOAD
          MOVLW B'00000111'   		;LOAD
          MOVWF	OPTION_REG		   	;WDT = 128
		  MOVLW	B'00010011'			;LOADING A/D FOSC/8 RBO INPUT
		  MOVWF	ANSEL        		;LOAD
          BCF   STATUS, RP0       	;BACK TO PAGE  
  		  CLRF	GPIO				;CLEAR GPIO
          MOVLW B'10000001'  		;MAKING RPO A/D  10 BIT
          MOVWF ADCON0             	;VREF = VDD GP0 ANALOG
		  MOVLW	B'11100000'			;TURN ON INTERRUPT TMRO
		  MOVWF	INTCON
		  MOVLW	.10					;LOAD MINUTES
	  	  MOVWF	MINUTES
		  MOVLW	.60					;LOAD SECONDS
		  MOVWF	SECONDS
		  MOVLW	.16
		  MOVWF	TIMER1
;********************TESTING**********************
TEST

;:*************************************************


START 	CALL	TURN_ON				;TURN ON POWER
S1		CALL 	MEASURE_BATTERY		;MEASURE BATTERY
		CALL	MEASURE_SOLAR		;MEASURE SOLAR
		MOVF	SOLAR_HI,0			;TRANSFERRING RESULTS FOR SUBTRACTION
		MOVWF	ACCbMID
		MOVF	SOLAR_LOW,0
		MOVWF	ACCbLO
		MOVF	BATTERY_HI,0
		MOVWF	ACCaMID
		MOVF	BATTERY_LOW,0
		MOVWF	ACCaLO
		CLRF	ACCaHI
		CLRF	ACCbHI
		CALL	Sub24				;PERFORM SUBTRACTION
		MOVLW	.255				;CHECKING TO SEE OF SOLAR IS LOWER THAN BATTERY
		SUBWF	ACCbHI,0
		BTFSS	3,2
		GOTO	S2
		CALL	TURN_OFF
		GOTO	S1

S2		MOVLW	.0					;IF ABOVE 1.25 VOLTS 20 AMPS
		SUBWF	ACCbHI,0
		BTFSC	3,2
		GOTO	START

ALARM	CALL	TURN_OFF			;OVER AMPERAGE
		MOVLW	.5					;DO 5 TIMES
		MOVWF	COUNTER3
ALARM1	BSF		RED					;BLINK RED LED
		BCF		GREEN
		DECFSZ	COUNTER1,1
		GOTO	ALARM1
		DECFSZ	COUNTER2,1
		GOTO	ALARM1
ALARM2	BCF		RED
		BCF		GREEN
		DECFSZ	COUNTER1,1
		GOTO	ALARM2
		DECFSZ	COUNTER2,1
		GOTO	ALARM2
		DECFSZ	COUNTER3,1
		GOTO	ALARM1
		GOTO	START

TURN_OFF
		BCF		POWER					;TURN OFF
		BSF		GREEN					;SHOW GREEN
		BCF		RED
		RETURN
TURN_ON
		BSF		POWER					;TURN ON 
		BCF		GREEN					;SHOW RED
		BSF		RED
		RETURN 

	
MEASURE_BATTERY

        MOVLW	B'10000101'  		;MAKING RPO A/D  10 BIT GPIO1
        MOVWF	ADCON0             	;VREF = VDD GP0 ANALOG		BSF		ADCON0,1			;START A/D
		BSF		ADCON0,1			;START A/D
		BTFSC	ADCON0,1			;WAIT UNTIL FINISHED
		GOTO	$-1
		MOVF	ADRESH,0			;LOADING BATTERY HI
		MOVWF	BATTERY_HI
		BSF		STATUS,RP0			;CHANGE BANKS
		MOVF	ADRESL,0			;LOADING BATTERY LOW
		MOVWF	BATTERY_LOW
		BCF		STATUS,RP0			;CHANGE BANKS
		RETURN

MEASURE_SOLAR
        MOVLW	B'10000001'  		;MAKING RPO A/D  10 BIT GPIO0
        MOVWF	ADCON0             	;VREF = VDD GP0 ANALOG
		BSF		ADCON0,1			;START A/D
		BTFSC	ADCON0,1			;WAIT UNTIL FINISHED
		GOTO	$-1
		MOVF	ADRESH,0			;LOAD RESULTS
		MOVWF	SOLAR_HI
		BSF		STATUS,RP0			;CHANGE BANKS
		MOVF	ADRESL,0			;LOADING BATTERY LOW
		MOVWF	SOLAR_LOW
		BCF		STATUS,RP0			;CHANGE BANKS
		RETURN

;____________________________________________________________________________	
;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     24-bit Addition                                                    -*
;*-                                                                        -*
;*-     Uses ACCa and ACCb                                                 -*
;*-                                                                        -*
;*-     ACCa + ACCb -> ACCb                                                -*
;*--------------------------------------------------------------------------*
;****************************************************************************
Add24
        movf    ACCaLO,W
        addwf   ACCbLO,1          ;add low bytes
        btfsc   STATUS,C        ;add in carry if necessary
        goto    A2
A1      movf    ACCaMID,W
        addwf   ACCbMID,1         ;add mid bytes
        btfsc   STATUS,C        ;add in carry if necessary
        incf    ACCbHI,1
        movf    ACCaHI,W
        addwf   ACCbHI,1          ;add high bytes
        retlw   0
A2      incf    ACCbMID,1
        btfsc   STATUS,Z
        incf    ACCbHI,1
        goto    A1
;****************************************************************************
;*--------------------------------------------------------------------------*       
;*-     Subtraction ( 24 - 24 -> 24 )                                      -*
;*-                                                                        -*
;*-     Uses ACCa, ACCb, ACCd                                              -*
;*-                                                                        -*
;*-     ACCa -> ACCd,                                                      -*
;*-     2's complement ACCa,                                               -*
;*-     call Add24 ( ACCa + ACCb -> ACCb ),                                -*
;*-     ACCd -> ACCa                                                       -*
;*--------------------------------------------------------------------------*
;****************************************************************************
Sub24
        movf    ACCaHI,W                ;Transfer ACCa to ACCd
        movwf   ACCdHI
        movf    ACCaMID,W
        movwf   ACCdMID
        movf    ACCaLO,W
        movwf   ACCdLO
        call    compA                   ;2's complement ACCa
        call    Add24                   ;Add ACCa to ACCb
        movf    ACCdHI,W                ;Transfer ACCd to ACCa
        movwf   ACCaHI
        movf    ACCdMID,W
        movwf   ACCaMID
        movf    ACCdLO,W
        movwf   ACCaLO
        retlw   0
;____________________________________________________________________________
;****************************************************************************
;*--------------------------------------------------------------------------*
;*-     2's Complement                                                     -*
;*-                                                                        -*
;*-     Uses ACCa                                                          -*
;*-                                                                        -*
;*-     Performs 2's complement conversion on ACCa                         -*
;*--------------------------------------------------------------------------*
;****************************************************************************
compA    
        comf    ACCaLO,1          ;invert all bits in accumulator a
        comf    ACCaMID,1
        comf    ACCaHI,1
        incf    ACCaLO,1          ;add one to accumulator a
        btfsc   STATUS,Z
        incf    ACCaMID,1
        btfsc   STATUS,Z
        incf    ACCaHI,1
        retlw   0




		  END                      	;END OF PROGRAM
